import 'dart:convert';

import 'package:anadea_flutter_loggy/loggers/dio.dart';
import 'package:dio/dio.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:loggy/loggy.dart';

import 'json_viewer/flutter_json_viewer.dart';

/// Used for dio records list on the [LogsScreen]
class DioRecordCard extends StatelessWidget {
  DioRecordCard({
    Key? key,
    required this.logRecord,
  })  : assert(logRecord.object is DioLogRecord),
        super(key: key);

  /// Must contains [DioLogRecord] in the [LogRecord.object] field
  final LogRecord logRecord;

  @override
  Widget build(BuildContext context) {
    final theme = Theme.of(context);
    final record = logRecord.object as DioLogRecord;

    return Card(
      elevation: 5,
      color: _getDioRecordColor(context, record),
      child: Padding(
        padding: const EdgeInsets.all(8.0),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Row(
              children: [
                Expanded(
                  child: Text(record.response?.statusCode.toString() ?? ''),
                ),
                IconButton(
                  onPressed: () => copyData(record),
                  icon: const Icon(Icons.copy),
                ),
              ],
            ),
            Text(
              record.toString(),
              style: theme.textTheme.bodyLarge,
            ),
            if (record.options != null) ...[
              if (record.options!.headers.isNotEmpty) ...[
                const Divider(),
                Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: Text(
                    'HEADERS',
                    style: theme.textTheme.bodyMedium,
                  ),
                ),
                JsonViewer(record.options!.headers),
              ],
              if (record.options!.data != null) ...[
                const Divider(),
                Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: Text(
                    'DATA',
                    style: theme.textTheme.bodyMedium,
                  ),
                ),
                if (record.options!.data is String)
                  JsonViewer(_decode(record.options!.data))
                else
                  JsonViewer(record.options!.data)
              ],
            ],
            if (record.response != null) ...[
              const Divider(),
              Padding(
                padding: const EdgeInsets.all(8.0),
                child: Text(
                  'HEADERS',
                  style: theme.textTheme.bodyMedium,
                ),
              ),
              JsonViewer(record.response!.headers.map),
              const Divider(),
              Padding(
                padding: const EdgeInsets.all(8.0),
                child: Text(
                  'DATA',
                  style: theme.textTheme.bodyMedium,
                ),
              ),
              _buildResponseData(record),
            ],
            if (record.error != null) ...[
              const Divider(),
              Text(record.error!.message),
            ],
            const SizedBox(height: 10),
            Text(
              logRecord.time.toIso8601String().split('T')[1],
              style: theme.textTheme.caption,
            ),
          ],
        ),
      ),
    );
  }

  void copyData(DioLogRecord record) {
    Clipboard.setData(
      ClipboardData(
        text: 'Options: ${record.options.toString()}\n'
            'Error: ${record.error.toString()}\n'
            'Response: ${record.response.toString()}\n',
      ),
    );
  }

  Widget _buildResponseData(DioLogRecord record) {
    if (record.response!.data is String) {
      var data = _decode(record.response!.data);
      if (data is String) {
        return Text(data);
      }
      return JsonViewer(data);
    } else if (record.response!.data is Map) {
      return JsonViewer(record.response!.data);
    } else if (record.response!.data is FormData) {
      return const Text('Form data');
    } else {
      return const Text('Unknown data');
    }
  }

  _decode(String data) {
    if (data.isEmpty) {
      return null;
    }
    try {
      return json.decode(data);
    } catch (_) {
      return data;
    }
  }

  Color _getDioRecordColor(BuildContext context, DioLogRecord record) {
    final colorScheme = Theme.of(context).colorScheme;

    switch (record.type) {
      case DioLogRecordType.error:
        return colorScheme.errorContainer;

      case DioLogRecordType.request:
      case DioLogRecordType.response:
        return colorScheme.surface;
    }
  }
}
