import 'dart:collection';

import 'package:anadea_flutter_loggy/loggers/bloc.dart';
import 'package:anadea_flutter_loggy/loggers/dio.dart';
import 'package:anadea_flutter_loggy/loggers/navigation.dart';
import 'package:anadea_flutter_loggy/logs_screen/bloc_record_card.dart';
import 'package:anadea_flutter_loggy/logs_screen/navigation_record_card.dart';
import 'package:flutter/material.dart';
import 'package:flutter_loggy/flutter_loggy.dart';
import 'package:loggy/loggy.dart';

import 'dio_record_card.dart';

class WrongPrinterException implements Exception {
  WrongPrinterException();

  @override
  String toString() {
    return 'WrongPrinterException: Loggy printer is not set as StreamPrinter!';
  }
}

typedef LogRecordCardBuilder = Widget Function(
  BuildContext context,
  LogRecord record,
);

/// Screen with separated by logger names logs records
///
/// Use with [StreamPrinter]
class LogsScreen extends StatelessWidget {
  LogsScreen({
    Key? key,
    this.logLevel = LogLevel.all,
    this.onBackPressed,
    Map<Type, LogRecordCardBuilder> customRecordBuilders = const {},
  })  : customRecordBuilders = {
          ...customRecordBuilders,
          DioLogRecord: LogsScreen._buildDioRecord,
          NavigationLogRecord: LogsScreen._buildNavigationRecord,
          BlocLogRecord: LogsScreen._buildBlocRecord,
        },
        super(key: key);

  /// Minimum level for log records
  final LogLevel? logLevel;

  /// Custom bulders for speciffic [LogRecord] objects.
  final Map<Type, LogRecordCardBuilder> customRecordBuilders;

  /// Callback for [AppBar] leading button
  final VoidCallback? onBackPressed;

  @override
  Widget build(BuildContext context) {
    if (Loggy.currentPrinter is! StreamPrinter ||
        Loggy.currentPrinter == null) {
      throw WrongPrinterException();
    }
    final StreamPrinter printer = Loggy.currentPrinter as StreamPrinter;

    return StreamBuilder<List<LogRecord>>(
      stream: printer.logRecord,
      builder: (
        BuildContext context,
        AsyncSnapshot<List<LogRecord>> records,
      ) {
        if (!records.hasData) {
          return const Center(child: CircularProgressIndicator());
        }

        final SplayTreeMap<String, List<LogRecord>> groupedRecords =
            SplayTreeMap((a, b) => a.compareTo(b));
        for (final record in records.data!) {
          if (record.level.priority < logLevel!.priority) continue;

          if (!groupedRecords.containsKey(record.loggerName)) {
            groupedRecords[record.loggerName] = [];
          }
          groupedRecords[record.loggerName]!.add(record);
        }

        return DefaultTabController(
          length: groupedRecords.keys.length,
          child: Scaffold(
            appBar: AppBar(
              automaticallyImplyLeading: false,
              leading: onBackPressed == null
                  ? null
                  : BackButton(onPressed: onBackPressed),
              bottom: TabBar(
                isScrollable: true,
                tabs: [
                  for (final loggerName in groupedRecords.keys)
                    Tab(text: loggerName)
                ],
              ),
            ),
            body: TabBarView(
              children: [
                for (final loggerName in groupedRecords.keys)
                  _buildRecordList(context, groupedRecords, loggerName),
              ],
            ),
          ),
        );
      },
    );
  }

  ListView _buildRecordList(
    BuildContext context,
    SplayTreeMap<String, List<LogRecord>> groupedRecords,
    String loggerName,
  ) {
    return ListView(
      reverse: true,
      children: [
        for (final record in groupedRecords[loggerName]!)
          if (customRecordBuilders.containsKey(record.object.runtimeType))
            customRecordBuilders[record.object.runtimeType]!
                .call(context, record)
          else
            _DefaultLoggyItemWidget(record)
      ],
    );
  }

  static Widget _buildNavigationRecord(BuildContext context, LogRecord record) {
    return NavigationRecordCard(logRecord: record);
  }

  static Widget _buildDioRecord(BuildContext context, LogRecord record) {
    return DioRecordCard(logRecord: record);
  }

  static Widget _buildBlocRecord(BuildContext context, LogRecord record) {
    return BlocRecordCard(logRecord: record);
  }
}

class _DefaultLoggyItemWidget extends StatelessWidget {
  const _DefaultLoggyItemWidget(this.record, {Key? key}) : super(key: key);

  final LogRecord record;

  @override
  Widget build(BuildContext context) {
    final String timeStr = record.time.toIso8601String().split('T')[1];

    return Card(
      clipBehavior: Clip.hardEdge,
      color: _getLogColor(context),
      child: ListTile(
        title: Text(
          '${record.level.name.toUpperCase()} - $timeStr',
          style: Theme.of(context).textTheme.bodyLarge,
        ),
        subtitle: Text(
          record.message,
          style: Theme.of(context).textTheme.bodyMedium,
        ),
      ),
    );
  }

  Color _getLogColor(BuildContext context) {
    final colorScheme = Theme.of(context).colorScheme;

    switch (record.level) {
      case LogLevel.info:
        return colorScheme.surface;

      case LogLevel.debug:
        return colorScheme.surfaceVariant;

      case LogLevel.warning:
        return colorScheme.tertiaryContainer;

      case LogLevel.error:
        return colorScheme.errorContainer;
    }

    return Colors.white;
  }
}
